#include "StdAfx.h"
#include "HoldingListPlugIn2.h"

#define MY_PLUGIN_NAME      "Holding List"
#define MY_PLUGIN_VERSION   "1.0"
#define MY_PLUGIN_DEVELOPER "Gergely Csernak"
#define MY_PLUGIN_COPYRIGHT "Free to be distributed as source code"

const   int     TAG_ITEM_HOLDING_NAME       = 1 ;
const   int     TAG_ITEM_HOLDING_TIME       = 2 ;
const   int     TAG_ITEM_REMAINING_TIME     = 3 ;

const   int     TAG_FUNC_HOLDING_EDIT       = 1 ;   // for the TAGs
const   int     TAG_FUNC_HOLDING_WAIT       = 2 ;   // for the TAGs

const   int     TAG_FUNC_HOLDING_EDITOR     = 10 ;  // when editing the point name
const   int     TAG_FUNC_HOLDING_WAIT_LIST  = 11 ;  // for the popup list elements
const   int     TAG_FUNC_HOLDING_WAIT_CLEAR = 12 ;  // cnacel the wait by the popup


//---CHoldingListPlugIn------------------------------------------------

CHoldingListPlugIn :: CHoldingListPlugIn ()
    : EuroScopePlugIn :: CPlugIn ( EuroScopePlugIn :: COMPATIBILITY_CODE,
                                   MY_PLUGIN_NAME,
                                   MY_PLUGIN_VERSION,
                                   MY_PLUGIN_DEVELOPER,
                                   MY_PLUGIN_COPYRIGHT )
{
    // register my TAG items
    RegisterTagItemType ( "Holding point name",     TAG_ITEM_HOLDING_NAME ) ;
    RegisterTagItemType ( "Holding time",           TAG_ITEM_HOLDING_TIME ) ;
    RegisterTagItemType ( "Remaining holding time", TAG_ITEM_REMAINING_TIME ) ;

    // and also the functions behind
    RegisterTagItemFunction ( "Holding name editor", TAG_FUNC_HOLDING_EDIT ) ;
    RegisterTagItemFunction ( "Holding time popup",  TAG_FUNC_HOLDING_WAIT ) ;

    // register my AC list
    m_HoldingList = RegisterAcList ( "Holding list" ) ;

    // test if the defintions are loaded from the settings
    // and insert only if empty
    if ( m_HoldingList.GetColumnNumber () == 0 )
        {
        // fill in the default columns
        m_HoldingList.AddColumnDefinition ( "C/S", 6, false,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_TYPE_CALLSIGN,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO ) ;
        m_HoldingList.AddColumnDefinition ( "H/P", 6, false,
                                            MY_PLUGIN_NAME, TAG_ITEM_HOLDING_NAME,
                                            MY_PLUGIN_NAME, TAG_FUNC_HOLDING_EDIT,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO ) ;
        m_HoldingList.AddColumnDefinition ( "AFL", 4, true,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_TYPE_ALTITUDE,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO ) ;
        m_HoldingList.AddColumnDefinition ( "CFL", 4, true,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_TYPE_TEMP_ALTITUDE,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_TEMP_ALTITUDE_POPUP,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO ) ;
        m_HoldingList.AddColumnDefinition ( "TIM", 6, true,
                                            MY_PLUGIN_NAME, TAG_ITEM_HOLDING_TIME,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO ) ;
        m_HoldingList.AddColumnDefinition ( "REM", 6, true,
                                            MY_PLUGIN_NAME, TAG_ITEM_REMAINING_TIME,
                                            MY_PLUGIN_NAME, TAG_FUNC_HOLDING_WAIT,
                                            NULL, EuroScopePlugIn :: TAG_ITEM_FUNCTION_NO ) ;
        }// no columns from settings
}


//---~CHoldingListPlugIn-----------------------------------------------

CHoldingListPlugIn :: ~CHoldingListPlugIn ( void )
{
}


//---OnGetTagItem-------------------------------------------------------

void    CHoldingListPlugIn :: OnGetTagItem ( EuroScopePlugIn :: CAircraft Aircraft,
                                             int ItemCode,
                                             int TagData,
                                             char sItemString [ 16 ],
                                             int * pColorCode,
                                             COLORREF * pRGB,
                                             double * pFontSize )
{
    int     idx, sec ;


    // get the AC index
    if (( idx = _SelectAcIndex ( Aircraft.GetCallsign ())) < 0 )
        return ;

    // stitch by the code
    switch ( ItemCode )
        {
        case TAG_ITEM_HOLDING_NAME :
            strcpy ( sItemString, m_HoldingData [ idx ].m_PointName ) ;
            break ;

        case TAG_ITEM_HOLDING_TIME :

            // sec since entered
            sec = (int) ( time ( NULL ) - m_HoldingData [ idx ].m_Entered ) ;

            // format
            sprintf ( sItemString, "%02d:%02d", sec / 60, sec % 60 ) ;
            break ;

        case TAG_ITEM_REMAINING_TIME :

            // if has leave time
            if ( m_HoldingData [ idx ].m_Leave )
                {
                // if has time left
                if ( m_HoldingData [ idx ].m_Leave >= time ( NULL ))
                    sec = (int) ( m_HoldingData [ idx ].m_Leave - time ( NULL )) ;

                // out of time
                else
                    {
                    sec = (int) ( time ( NULL ) - m_HoldingData [ idx ].m_Leave ) ;
                    * pColorCode = EuroScopePlugIn :: TAG_COLOR_EMERGENCY ;
                    }

                // format
                sprintf ( sItemString, "%02d:%02d", sec / 60, sec % 60 ) ;
                }// has time specified

            break ;
        }// switch by the code
}


//---OnCompileCommand---------------------------------------------------

bool    CHoldingListPlugIn :: OnCompileCommand ( const char * sCommandLine )
{
    CHoldingData    data ;
    int             idx ;


    // if does not start with .hold
    if ( strncmp ( sCommandLine, ".hl", 3 ) || strlen ( sCommandLine ) < 5 )
        return false ;

    // save the tail
    CString cmd = sCommandLine + 4 ;
    cmd.MakeUpper () ;

    // show or hide the list
    if ( strcmp ( cmd, "SHOW" ) == 0 )
        {
        m_HoldingList.ShowAcList ( true ) ;
        return true ;
        }
    if ( strcmp ( cmd, "HIDE" ) == 0 )
        {
        m_HoldingList.ShowAcList ( false ) ;
        return true ;
        }

    // adding new plane
    if ( strncmp ( cmd, "ADD", 3 ) == 0 )
        {
        // cut
        cmd = cmd.Mid ( 4 ) ;

        // save the holding point name
        if (( idx = cmd.Find ( ' ' )) < 0 )
            idx = cmd.GetLength () ;
        data.m_PointName = cmd.Left ( idx ) ;
        cmd = cmd.Mid ( idx + 1 ) ;

        // save the AC name
        data.m_Callsign = cmd ;

        // if no callsign, use the ASEL
        if ( data.m_Callsign.IsEmpty ())
            {
            EuroScopePlugIn :: CAircraft    ac = AircraftSelectASEL () ;
            if ( ! ac.IsValid ())
                return false ;
            data.m_Callsign = ac.GetCallsign () ;
            }// no callsign

        // if already there then delete
        if (( idx = _SelectAcIndex ( data.m_Callsign )) >= 0 )
            m_HoldingData.RemoveAt ( idx ) ;

        // get the AC
        EuroScopePlugIn :: CAircraft    ac = AircraftSelect ( data.m_Callsign ) ;
        if ( ! ac.IsValid ())
            return false ;

        // save
        data.m_Entered = time ( NULL ) ;
        data.m_Leave   = 0 ;
        data.m_Hard    = true ;
        m_HoldingData.Add ( data ) ;
        m_HoldingList.RemoveAcFromTheList ( ac ) ;
        m_HoldingList.AddAcToTheList ( ac ) ;
        return true ;
        }

    // removing plane
    if ( strncmp ( cmd, "DEL", 3 ) == 0 )
        {
        // cut
        cmd = cmd.Mid ( 4 ) ;

        // if no callsign the use the ASEL
        if ( cmd.IsEmpty ())
            {
            EuroScopePlugIn :: CAircraft    ac = AircraftSelectASEL () ;
            if ( ! ac.IsValid ())
                return false ;
            cmd = ac.GetCallsign () ;
            }

        // search
        if (( idx = _SelectAcIndex ( cmd )) < 0 )
            return false ;

        // remove
        m_HoldingData.RemoveAt ( idx ) ;

        // remove from the list too
        EuroScopePlugIn :: CAircraft    ac = AircraftSelect ( cmd ) ;
        if ( ac.IsValid ())
            m_HoldingList.RemoveAcFromTheList ( ac ) ;

        return true ;
        }

    // wait time
    if ( strncmp ( cmd, "WAIT", 4 ) == 0 )
        {
        // cut
        cmd = cmd.Mid ( 5 ) ;

        // save the minutes
        int sec = 60 * atoi ( cmd ) ;

        // can not enter 0
        if ( sec <= 0 )
            return false ;

        // get the callsign
        if (( idx = cmd.Find ( ' ' )) < 0 )
            return false ;
        data.m_Callsign = cmd.Mid ( idx + 1 ) ;

        // if no callsign the use the ASEL
        if ( data.m_Callsign.IsEmpty ())
            {
            EuroScopePlugIn :: CAircraft    ac = AircraftSelectASEL () ;
            if ( ! ac.IsValid ())
                return false ;
            data.m_Callsign = ac.GetCallsign () ;
            }

        // if no AC in the list
        if (( idx = _SelectAcIndex ( data.m_Callsign )) < 0 )
            return false ;

        // save
        m_HoldingData [ idx ].m_Leave = time ( NULL ) + sec ;
        return true ;
        }

    // show/hide 
    return false ;
}


//---_SelectAcIndex-----------------------------------------------------

int     CHoldingListPlugIn :: _SelectAcIndex ( const char * sCallsign )
{
    // search
    int     i ;
    for ( i = 0 ;
          i < m_HoldingData.GetSize () && m_HoldingData [ i ].m_Callsign.CompareNoCase ( sCallsign ) ;
          i ++ ) ;
    if ( i < m_HoldingData.GetSize ())
        return i ;
    else
        return -1 ;
}

//---OnAircraftControllerAssignedDataUpdate----------------------------

void    CHoldingListPlugIn :: OnAircraftControllerAssignedDataUpdate ( EuroScopePlugIn :: CAircraft Aircraft,
                                                                       int DataType )
{
    CHoldingData    data ;
    int             idx ;
    CString         str ;
    int             sec ;


    // if not scratch pad message
    if ( DataType != EuroScopePlugIn :: CTR_DATA_TYPE_SCRATCH_PAD_STRING )
        return ;

    EuroScopePlugIn :: CAircraftControllerAssignedData ctrdata = Aircraft.GetControllerAssignedData () ;

    // if changed to HOLD
    str = ctrdata.GetScratchPadString () ;
    str.MakeUpper () ;
    if ( strncmp ( str, "HOLD", 4 ) == 0 )
        {
        // fill in the data
        data.m_PointName = ctrdata.GetDirectToPointName () ;
        data.m_Callsign  = Aircraft.GetCallsign () ;

        // if already there then delete
        if (( idx = _SelectAcIndex ( data.m_Callsign )) >= 0 )
            m_HoldingData.RemoveAt ( idx ) ;

        // if has leave data
        if (( sec = atoi ( str.Mid ( 4 )) * 60 ) > 0 )
            data.m_Leave = time ( NULL ) + sec ;
        else
            data.m_Leave = 0 ;

        // save
        data.m_Entered = time ( NULL ) ;
        data.m_Hard    = false ;
        m_HoldingData.Add ( data ) ;
        m_HoldingList.RemoveAcFromTheList ( Aircraft ) ;
        m_HoldingList.AddAcToTheList ( Aircraft ) ;
        } // HOLD

    // if empty
    if ( * ctrdata.GetScratchPadString () == 0 )
        {
        // search for it
        if (( idx = _SelectAcIndex ( Aircraft.GetCallsign ())) < 0 )
            return ;

        // if soft insert
        if ( ! m_HoldingData [ idx ].m_Hard )
            {
            m_HoldingData.RemoveAt ( idx ) ;
            m_HoldingList.RemoveAcFromTheList ( Aircraft ) ;
            }
        }// empty
}

//---OnFunctionCall-----------------------------------------------------

void    CHoldingListPlugIn :: OnFunctionCall ( int FunctionId,
                                               const char * sItemString,
                                               POINT Pt,
                                               RECT Area )
{
    EuroScopePlugIn :: CAircraft    ac ;
    int                             i ;
    CString                         str ;


    // get the ASEL AC
    ac = AircraftSelectASEL () ;
    if ( ! ac.IsValid ())
        return ;

    // select it from  my list
    int idx = _SelectAcIndex ( ac.GetCallsign ()) ;
    if ( idx < 0 )
        return ;

    // switch by the function ID
    switch ( FunctionId )
        {
        case TAG_FUNC_HOLDING_EDIT : // TAG function

            // open a popup editor there
            OpenPopupEdit ( Area,
                            TAG_FUNC_HOLDING_EDITOR,
                            m_HoldingData [ idx ].m_PointName ) ;
            break ;

        case TAG_FUNC_HOLDING_EDITOR : // when finished editing

            // simply save the value
            m_HoldingData [ idx ].m_PointName = sItemString ;
            m_HoldingData [ idx ].m_PointName.MakeUpper () ;
            m_HoldingData [ idx ].m_Hard      = true ;
            break ;

        case TAG_FUNC_HOLDING_WAIT : // TAG function

            // start a popup list
            OpenPopupList ( Area, "Wait", 1 ) ;

            // add minutes from 90 to 30 by 10
            for ( i = 90 ; i > 30 ; i -= 10 )
                {
                str.Format ( "%d", i ) ;
                AddPopupListElement ( str, "", TAG_FUNC_HOLDING_WAIT_LIST ) ;
                }
            // add minutes from 30 to 10 by 5
            for ( ; i > 10 ; i -= 5 )
                {
                str.Format ( "%d", i ) ;
                AddPopupListElement ( str, "", TAG_FUNC_HOLDING_WAIT_LIST ) ;
                }
            // add minutes from 10 to 1 by one
            for ( ; i > 0 ; i -- )
                {
                str.Format ( "%d", i ) ;
                AddPopupListElement ( str, "", TAG_FUNC_HOLDING_WAIT_LIST, i == 10 ) ;
                }

            // finally add a fixed element to clear the wait time
            AddPopupListElement ( "Clear", "",
                                  TAG_FUNC_HOLDING_WAIT_CLEAR,
                                  false,
                                  EuroScopePlugIn :: POPUP_ELEMENT_NO_CHECKBOX,
                                  m_HoldingData [ idx ].m_Leave == 0,
                                  true ) ;
            break ;

        case TAG_FUNC_HOLDING_WAIT_LIST : // a value is selected from the popup list

            // save the time when to leave
            m_HoldingData [ idx ].m_Leave = time ( NULL ) + 60 * atoi ( sItemString ) ;
            break ;

        case TAG_FUNC_HOLDING_WAIT_CLEAR : // clear the waiting time

            // simply clear
            m_HoldingData [ idx ].m_Leave = 0 ;
            break ;

        }// switch by the function ID
}
