#include "StdAfx.h"
#include "OccScreen.h"
#include "resource.h"

const   COLORREF    COLOR_BLACK             = RGB (   0,   0,   0 ) ;
const   COLORREF    COLOR_OCC_BACK          = RGB ( 220, 220, 220 ) ;
const   COLORREF    COLOR_OCC_HILITE        = RGB ( 220, 255, 255 ) ;

static int _a_multipliers_f [] = { 7, 2, 5, 0 } ;
static int _a_widths_f      [] = { 1, 1, 3, 0 } ;

static int _a_multipliers [] = { 3, 7, 2, 5, 7, 2, 5, 5, 0 } ;
static int _a_widths      [] = { 3, 1, 1, 3, 1, 1, 3, 3, 0 } ;

const   int     OBJTYPE_OCCTEXT_CALLSIGN = 1 ;


//---COccScreen--------------------------------------------------------

COccScreen :: COccScreen ( void )
{
    m_FinalLines = 2 ;
    m_DispLines  = 27 ;
    m_Valid      = false ;
    m_TopEmpty = m_BottomEmpty = m_LeftEmpty = m_RightEmpty = 0 ;
    m_MovingObjectType = 0 ;
}


//---~COccScreen-------------------------------------------------------

COccScreen :: ~COccScreen ( void )
{
}


//---OnAsrContentLoaded------------------------------------------------

void    COccScreen :: OnAsrContentLoaded ( bool Loaded )
{
    char        s_line [ 256 ] ;
    const char  * p_help, * p_help2 ;
    CString     str ;
    bool        first_coord = true ;


    // get the values
    if ( GetDataFromAsr ( "AIRPORT" ) == NULL )
        {
        AfxMessageBox ( "No AIRPORT name defined in the ASR file" ) ;
        return ;
        }

    // final lines
    if (( p_help = GetDataFromAsr ( "FINAL" )) != NULL )
        {
        m_FinalLines = atoi ( p_help ) ;
        if ( m_FinalLines < 1 )
            m_FinalLines = 1 ;
        if ( m_FinalLines > 4 )
            m_FinalLines = 4 ;
        }

    // max lines
    if (( p_help = GetDataFromAsr ( "LINES" )) != NULL )
        {
        m_DispLines = atoi ( p_help ) ;
        if ( m_DispLines < 5 )
            m_DispLines = 5 ;
        if ( m_DispLines >= 40 )
            m_DispLines = 39 ;
        }

    // empty area
    if (( p_help = GetDataFromAsr ( "TOPEMPTY" )) != NULL )
        {
        m_TopEmpty = atoi ( p_help ) ;
        if ( m_TopEmpty < 0 )
            m_TopEmpty = 0 ;
        if ( m_TopEmpty >= 500 )
            m_TopEmpty = 500 ;
        }
    if (( p_help = GetDataFromAsr ( "BOTTOMEMPTY" )) != NULL )
        {
        m_BottomEmpty = atoi ( p_help ) ;
        if ( m_BottomEmpty < 0 )
            m_BottomEmpty = 0 ;
        if ( m_BottomEmpty >= 500 )
            m_BottomEmpty = 500 ;
        }
    if (( p_help = GetDataFromAsr ( "LEFTEMPTY" )) != NULL )
        {
        m_LeftEmpty = atoi ( p_help ) ;
        if ( m_LeftEmpty < 0 )
            m_LeftEmpty = 0 ;
        if ( m_LeftEmpty >= 500 )
            m_LeftEmpty = 500 ;
        }
    if (( p_help = GetDataFromAsr ( "RIGHTEMPTY" )) != NULL )
        {
        m_RightEmpty = atoi ( p_help ) ;
        if ( m_RightEmpty < 0 )
            m_RightEmpty = 0 ;
        if ( m_RightEmpty >= 500 )
            m_RightEmpty = 500 ;
        }

    // separators
    for ( int i = 1 ; ; i ++ )
        {
        // variable name
        str.Format ( "SEPARATOR%d", i ) ;

        // get it from ASR
        if (( p_help = GetDataFromAsr ( str )) == NULL )
            break ;

        // save
        m_SplitLines.Add ( atoi ( p_help )) ;
        }

    // stands
    for ( int i = 1 ; ; i ++ )
        {
        // variable name
        str.Format ( "STAND%d", i ) ;

        // get it from ASR
        if (( p_help = GetDataFromAsr ( str )) == NULL )
            break ;

        // copy the value for examining
        strncpy ( s_line, p_help, 256 ) ;
        s_line [ 255 ] = 0 ;

        // try to fill in the stand
        COccStand   stand ;

        // pos 1 = gate number
        if (( p_help = strtok ( s_line, ":" )) == NULL )
            continue ;
        stand.m_GateNumber = p_help ;

        // pos 2 = gate name
        if (( p_help = strtok ( NULL, ":" )) == NULL )
            continue ;
        stand.m_Name = p_help ;

        // pos 3 = gate index X, pos 4 = gate index Y
        if (( p_help = strtok ( NULL, ":" )) == NULL )
            continue ;
        stand.m_IndexX = atoi ( p_help ) ;
        if (( p_help = strtok ( NULL, ":" )) == NULL )
            continue ;
        stand.m_IndexY = atoi ( p_help ) ;

        // pos 5, 6 latitude and longitude
        if (( p_help = strtok ( NULL, ":" )) == NULL )
            continue ;
        if (( p_help2 = strtok ( NULL, ":" )) == NULL )
            continue ;
        if ( ! stand.m_aCoords [ 0 ].LoadFromStrings ( p_help2, p_help ))
            continue ;
        stand.m_aCoordsSet [ 0 ] = true ;

        // extend the area
        if ( first_coord || m_AirportExtent.m_LeftDown.m_Latitude > stand.m_aCoords [ 0 ].m_Latitude )
            m_AirportExtent.m_LeftDown.m_Latitude = stand.m_aCoords [ 0 ].m_Latitude ;
        if ( first_coord || m_AirportExtent.m_LeftDown.m_Longitude > stand.m_aCoords [ 0 ].m_Longitude )
            m_AirportExtent.m_LeftDown.m_Longitude = stand.m_aCoords [ 0 ].m_Longitude ;
        if ( first_coord || m_AirportExtent.m_RightUp.m_Latitude < stand.m_aCoords [ 0 ].m_Latitude )
            m_AirportExtent.m_RightUp.m_Latitude = stand.m_aCoords [ 0 ].m_Latitude ;
        if ( first_coord || m_AirportExtent.m_RightUp.m_Longitude < stand.m_aCoords [ 0 ].m_Longitude )
            m_AirportExtent.m_RightUp.m_Longitude = stand.m_aCoords [ 0 ].m_Longitude ;
        first_coord = false ;

        // pos 7 and 8 are optional
        if (( p_help = strtok ( NULL, ":" )) != NULL &&
            ( p_help2 = strtok ( NULL, ":" )) == NULL &&
            stand.m_aCoords [ 1 ].LoadFromStrings ( p_help2, p_help ))
            stand.m_aCoordsSet [ 2 ] = true ;

        // search for the place
        int     j ;
        for ( j = m_Stands.GetSize () ;
              j > 0 && ( stand.m_IndexY < m_Stands [ j - 1 ].m_IndexY ||
                         ( stand.m_IndexY == m_Stands [ j - 1 ].m_IndexY &&
                           stand.m_IndexX < m_Stands [ j - 1 ].m_IndexX ) ) ;
              j -- )
            {
            COccStand   helpstand = m_Stands [ j - 1 ] ;
            m_Stands.SetAtGrow ( j, helpstand ) ;
            }

        // save
        m_Stands.SetAtGrow ( j, stand ) ;
        }// loop over the stands

    // if no coords at all
    if ( first_coord )
        {
        AfxMessageBox ( "No stand definition is loaded" ) ;
        return ;
        }

    // ok, valid, loaded
    m_Valid = true ;

    // extend the whole area by 500 ft
    m_AirportExtent.m_LeftDown.m_Latitude  -= 0.25 / 60.0 ;
    m_AirportExtent.m_LeftDown.m_Longitude -= 0.25 / 60.0 ;
    m_AirportExtent.m_RightUp.m_Latitude   += 0.25 / 60.0 ;
    m_AirportExtent.m_RightUp.m_Longitude  += 0.25 / 60.0 ;

    // bigger extent
    m_AirportBiggerExtent = m_AirportExtent ;
    m_AirportBiggerExtent.m_LeftDown.m_Latitude  -= 20.0 / 60.0 ;
    m_AirportBiggerExtent.m_LeftDown.m_Longitude -= 20.0 / 60.0 ;
    m_AirportBiggerExtent.m_RightUp.m_Latitude   += 20.0 / 60.0 ;
    m_AirportBiggerExtent.m_RightUp.m_Longitude  += 20.0 / 60.0 ;
}


//---OnRefresh----------------------------------------------------------

void    COccScreen :: OnRefresh ( HDC hDC,
                                  int Phase )
{
    CBrush      white_brush ( COLOR_OCC_BACK ) ;
    int         dec, i ;
    POSITION    pos, pos2 ;


    // only in first phase
    if ( Phase != EuroScopePlugIn :: REFRESH_PHASE_BEFORE_TAGS )
        return ;

    // I am using MFC, so load the DC in to MFC structure
    CDC     dc ;
    dc.Attach ( hDC ) ;

    // get the area that is not coevered by the toolbar and the chat
    CRect   rect1 = GetRadarArea () ;
    CRect   rect2 = GetChatArea () ;
    rect1.bottom = rect2.top ;
    rect1.top    += m_TopEmpty ;
    rect1.bottom -= m_BottomEmpty ;
    rect1.left   += m_LeftEmpty ;
    rect1.right  -= m_RightEmpty ;

    // transparency
    dc.SetBkMode ( TRANSPARENT ) ;

    // first clear the whole area
    dc.FillRect ( & rect1, & white_brush ) ;
    dc.SetBkColor ( COLOR_OCC_BACK ) ;
    dc.SetTextColor ( COLOR_BLACK ) ;

    // decide the sizes
    dec  = 2 * 3 ;                          // the top and bottom line
    dec += m_FinalLines - 1 ;               // final separator
    dec += 3 ;                              // final end and top of lines
    dec += m_DispLines ;                    // display separators
    dec += 2 * m_SplitLines.GetSize () ;    // wide splitters adds

    // decide the m_Height of one lines, but minimu 10 pixels
    m_Height = ( rect1.Height () - dec ) / ( m_FinalLines + m_DispLines ) ;
    if ( m_Height < 10 )
        m_Height = 10 ;

    // m_Width
    dec  = 0 ;
    dec += 9 * 3 ;                          // border
    dec += 8 ;                              // single separators

    // decide the m_Widths
    m_Width = ( rect1.Width () - dec ) / 2 ;
    m_Width /= 3 + 7 + 2 + 5 + 7 + 2 + 5 + 5 ;

    // reset ACs at the gate positions
    for ( i = 0 ; i < m_Stands.GetSize () ; i ++ )
        m_Stands [ i ].m_ACsAtGate.SetSize ( 0, 5 ) ;

    // reset finalists
    m_Finals.SetSize ( 0, 10 ) ;

    // test vinicity ACs
    pos = m_VinicityAcList.GetHeadPosition () ;
    while ( pos )
        {
        // save and next
        pos2 = pos ;
        COccAircraft    & occac = m_VinicityAcList.GetNext ( pos ) ; ;

        // if disconnected
        EuroScopePlugIn :: CAircraft ac = GetPlugIn () -> AircraftSelect ( occac.m_Callsign ) ;
        if ( ! ac.IsValid ())
            m_VinicityAcList.RemoveAt ( pos2 ) ;

        // if at gate
        else if ( m_VinicityAcList.GetAt ( pos2 ).m_GateIndex >= 0 )
            m_Stands [ occac.m_GateIndex ].m_ACsAtGate.Add ( pos2 ) ;

        // if arriving
        else if ( ( occac.m_State == OCC_STATE_ON_FINAL ||
                    occac.m_State == OCC_STATE_TAXIING_TO_GATE ) )
            {
            COccFinalPositions  occfinal ;

            // save the pointer
            occfinal.m_ACPosition = pos2 ;
            occfinal.m_GateIndex  = _SearchForAllocation ( occac.m_Callsign ) ;
            m_Finals.Add ( occfinal ) ;
            }// arriving
        }// test vinicity ACs

    // draw the grids
    _DrawGridBorders ( & dc, rect1 ) ;

    // draw the texts
    _DrawGridTexts ( & dc, rect1 ) ;

    // detach
    dc.Detach () ;
}


//---OnAircraftPositionUpdate------------------------------------------

void    COccScreen :: OnAircraftPositionUpdate ( EuroScopePlugIn :: CAircraft Aircraft )
{
    EuroScopePlugIn :: CAircraftPositionData    acpos = Aircraft.GetPosition () ;
    int                 gate ;
    POSITION            pos ;
    COccAircraft        occac ;


    // if not in the area of interests
    if ( acpos.GetPressureAltitude () > 10000 ||
         acpos.GetPosition ().m_Latitude  < m_AirportBiggerExtent.m_LeftDown.m_Latitude ||
         acpos.GetPosition ().m_Latitude  > m_AirportBiggerExtent.m_RightUp.m_Latitude ||
         acpos.GetPosition ().m_Longitude < m_AirportBiggerExtent.m_LeftDown.m_Longitude ||
         acpos.GetPosition ().m_Longitude > m_AirportBiggerExtent.m_RightUp.m_Longitude )
        {
        // if in the list
        if (( pos = _SelectAcFromVinicityList ( Aircraft.GetCallsign ())) != NULL )
            m_VinicityAcList.RemoveAt ( pos ) ;

        return ;
        }

    // get the description from the list
    pos = _SelectAcFromVinicityList ( Aircraft.GetCallsign ()) ;

    // if first time in the vinicity
    if ( pos == NULL )
        {
        // fill in the structure
        occac.m_Callsign       = Aircraft.GetCallsign () ;
        occac.m_StateChanged   = time ( NULL ) ;
        occac.m_AcceptedAtGate = false ;

        // if in the air
        if ( Aircraft.GetGS () > 50 )
            {
            // if not coming to here
            if ( m_AirportName.CompareNoCase ( Aircraft.GetFlightPlan ().GetDestination ()) == 0 )
                return ;

            // on final anyway
            occac.m_State = OCC_STATE_ON_FINAL ;
            }
        else
            // departing from here
            occac.m_State = OCC_STATE_AT_GATE_DEPARTING ;
        }// first time in the vinicity

    // found in the list
    else
        {
        // save and remove
        occac = m_VinicityAcList.GetAt ( pos ) ;
        m_VinicityAcList.RemoveAt ( pos ) ;
        }
            
    // if the plane is on the ground and not moving and inside the local area
    // and really close to a gate
    if ( acpos.GetPressureAltitude () < 800 &&
         Aircraft.GetGS () < 5 &&
         acpos.GetPosition ().m_Latitude  >= m_AirportBiggerExtent.m_LeftDown.m_Latitude &&
         acpos.GetPosition ().m_Latitude  <= m_AirportBiggerExtent.m_RightUp.m_Latitude &&
         acpos.GetPosition ().m_Longitude >= m_AirportBiggerExtent.m_LeftDown.m_Longitude &&
         acpos.GetPosition ().m_Longitude <= m_AirportBiggerExtent.m_RightUp.m_Longitude &&
         ( gate = _TestIfCoordAtGate ( acpos.GetPosition ())) > 0 )
        {
        // save the current gate index
        occac.m_GateIndex = gate ;

        // if assigned to here then accept to the gate
        if ( _TestIfAllocatedToHere ( occac.m_Callsign, gate ))
            occac.m_AcceptedAtGate = true ;

        // if state is not at the gate
        if ( occac.m_State == OCC_STATE_TAXIING_TO_RW ||
             occac.m_State == OCC_STATE_TAKING_OFF )
            occac.SetNewState ( OCC_STATE_AT_GATE_DEPARTING ) ;
        else if ( occac.m_State == OCC_STATE_ON_FINAL ||
                  occac.m_State == OCC_STATE_TAXIING_TO_GATE )
            occac.SetNewState ( OCC_STATE_AT_GATE_ARRIVED ) ;
        }// AC is at the gate

    // not at any gate
    else
        {
        // delete gate index
        occac.m_GateIndex = -1 ;

        // if just left the gate
        if ( occac.m_State == OCC_STATE_AT_GATE_DEPARTING ||
             occac.m_State == OCC_STATE_AT_GATE_PUSHING_BACK )
            occac.SetNewState ( OCC_STATE_TAXIING_TO_RW ) ;
        else if ( occac.m_State == OCC_STATE_AT_GATE_ARRIVED )
            occac.SetNewState ( OCC_STATE_TAXIING_TO_GATE ) ;

        // if landed
        if ( occac.m_State == OCC_STATE_ON_FINAL && Aircraft.GetGS () < 50 )
            occac.SetNewState ( OCC_STATE_TAXIING_TO_GATE ) ;

        // if took of
        if ( ( occac.m_State == OCC_STATE_AT_GATE_DEPARTING ||
               occac.m_State == OCC_STATE_AT_GATE_PUSHING_BACK ||
               occac.m_State == OCC_STATE_TAXIING_TO_RW ) &&
               Aircraft.GetGS () > 50 )
            occac.SetNewState ( OCC_STATE_TAKING_OFF ) ;

        // go around
        if ( occac.m_State == OCC_STATE_TAXIING_TO_GATE && Aircraft.GetGS () > 50 )
            occac.SetNewState ( OCC_STATE_ON_FINAL ) ;
        }

    // calculate the distance
    occac.m_Dist = Aircraft.GetDistanceToDestination () ;

    // insert to the list again maintinag distance order
    pos = m_VinicityAcList.GetHeadPosition () ;
    while ( pos )
        {
        // if farer away
        if ( m_VinicityAcList.GetAt ( pos ).m_Dist > occac.m_Dist )
            break ;

        // next
        m_VinicityAcList.GetNext ( pos ) ;
        }

    // insert before the found or to the tail
    if ( pos )
        m_VinicityAcList.InsertBefore ( pos, occac ) ;
    else
        m_VinicityAcList.AddTail ( occac ) ;
}


//---OnCompileCommand--------------------------------------------------

bool    COccScreen :: OnCompileCommand ( const char * sCommandLine )
{
    int                             idx ;
    const char                      * p_call ;
    EuroScopePlugIn :: CAircraft    ac ;
    CString                         gate ;


    // if occ command
    if ( strncmp ( sCommandLine, ".occ", 4 ) != 0 )
        return false ;
    sCommandLine += 4 ;

    // if arrive
    if ( strncmp ( sCommandLine, "arr ", 4 ) == 0 )
        {
        // save gate number
        if (( p_call = strchr ( sCommandLine + 5, ' ' )) == NULL )
            return false ;
        gate = sCommandLine + 4 ;
        if ( gate.IsEmpty ())
            return false ;

        // allocate for arrive
        _ArriveToGate ( gate, p_call + 1 ) ;
        return true ;
        }// arrive

    // if depart
    else if ( strncmp ( sCommandLine, "dep ", 4 ) == 0 )
        {
        // AC
        if (( p_call = strchr ( sCommandLine + 5, ' ' )) == NULL )
            p_call = " *" ;

        // save gate number
        gate = sCommandLine + 4 ;
        if ( gate.IsEmpty ())
            return false ;

        // allocate for arrive
        _DepartFromGate ( gate, p_call + 1 ) ;
        return true ;
        }// depart

    // free
    else if ( strncmp ( sCommandLine, "free ", 5 ) == 0 )
        {
        // gate
        gate = sCommandLine + 5 ;
        if ( gate.IsEmpty ())
            return false ;

        // try to interpret as gate number
        if (( idx = _SelectGate ( gate )) >= 0 )
            {
            // simple delete the first allocation
            if ( m_Stands [ idx ].m_Allocations.GetSize ())
                m_Stands [ idx ].m_Allocations.RemoveAt ( 0 ) ;
            }

        // delete all allocation of the AC
        else
            _DeleteAllocations ( sCommandLine + 5 ) ;
        return true ;
        }// release

    // push
    else if ( strncmp ( sCommandLine, "push ", 5 ) == 0 )
        {
        POSITION    pos ;

        // select the AC from the vinicity
        if (( pos = _SelectAcFromVinicityList ( sCommandLine + 5 )) != NULL )
            {
            COccAircraft    & occac = m_VinicityAcList.GetAt ( pos ) ;

            // if at the gate then start pushing back
            if ( occac.m_State == OCC_STATE_AT_GATE_DEPARTING ||
                 occac.m_State == OCC_STATE_AT_GATE_ARRIVED )
                occac.SetNewState ( OCC_STATE_AT_GATE_PUSHING_BACK ) ;
            }
        return true ;
        }// push

    // gate
    else if ( strncmp ( sCommandLine, "gate ", 5 ) == 0 )
        {
        // extract callsign
        p_call = strchr ( sCommandLine + 6, ' ' ) ;

        // gate
        gate = sCommandLine + 5 ;
        if ( gate.IsEmpty ())
            return false ;

        // save gate number
        if (( idx = _SelectGate ( gate )) < 0 )
            return false ;

        // if no callsign then delete
        if ( p_call == NULL )
            {
            m_Stands [ idx ].m_aCoordsSet [ 1 ] = false ;
            return false ;
            }

        // selet the AC
        CString str = p_call + 1 ;
        str.MakeUpper () ;
        ac = GetPlugIn () -> AircraftSelect ( str ) ;
        if ( ! ac.IsValid ())
            return false ;

        // get the AC, and save the coords
        m_Stands [ idx ].m_aCoords [ 1 ] = ac.GetPosition ().GetPosition () ;
        m_Stands [ idx ].m_aCoordsSet [ 1 ] = true ;
        return true ;
        }// gate

    // empty areas
    else if ( strncmp ( sCommandLine, "left ", 5 ) == 0 )
        {
        m_LeftEmpty = atoi ( sCommandLine + 5 ) ;
        return true ;
        }
    else if ( strncmp ( sCommandLine, "right ", 6 ) == 0 )
        {
        m_RightEmpty = atoi ( sCommandLine + 6 ) ;
        return true ;
        }
    else if ( strncmp ( sCommandLine, "top ", 4 ) == 0 )
        {
        m_TopEmpty = atoi ( sCommandLine + 4 ) ;
        return true ;
        }
    else if ( strncmp ( sCommandLine, "bottom ", 7 ) == 0 )
        {
        m_BottomEmpty = atoi ( sCommandLine + 7 ) ;
        return true ;
        }

    return false ;
}


//---OnMoveScreenObject------------------------------------------------

void    COccScreen :: OnMoveScreenObject ( int ObjectType,
                                           const char * sObjectId,
                                           POINT Pt,
                                           RECT Area,
                                           bool Released )
{
    int     i ;


    // simple save
    m_MovingObjectType = Released ? 0 : ObjectType ;
    m_MovingObjectId   = sObjectId ;
    m_MovingArea       = Area ;

    // and we need an update
    RequestRefresh () ;

    // test if the mouse is over a stand
    m_MovingOverStandIndex = -1 ;
    for ( i = 0 ;
          i < m_Stands.GetSize () && ! m_Stands [ i ].m_MainArea.PtInRect ( Pt ) ;
          i ++ ) ;
    if ( i == m_Stands.GetSize ())
        return ;

    // save the index
    m_MovingOverStandIndex = i ;

    // if we are just releasing, then allocate for arrival
    if ( Released )
        {
        POSITION    pos = _SelectAcFromVinicityList ( sObjectId ) ;
        if ( pos == NULL )
            return ;
        COccAircraft    & occac = m_VinicityAcList.GetAt ( pos ) ;

        // departing
        if ( occac.m_State == OCC_STATE_AT_GATE_DEPARTING )
            _DepartFromGate ( m_Stands [ i ].m_GateNumber, sObjectId ) ;

        // arriving
        if ( occac.m_State == OCC_STATE_ON_FINAL ||
             occac.m_State == OCC_STATE_TAXIING_TO_GATE )
            _ArriveToGate ( m_Stands [ i ].m_GateNumber, sObjectId ) ;
        }
}


//---_DrawGridBorders--------------------------------------------------

void    COccScreen :: _DrawGridBorders ( CDC * pDC,
                                         const CRect & DisplayArea )
{
    CPen    pen0 ( 0, 0, COLOR_BLACK ), pen3 ( 0, 3, COLOR_BLACK ) ;
    CPen    * p_old_pen ;
    int     i, j, k, l, m ;


    // top line
    int     y = DisplayArea.top + 1 ;
    p_old_pen = pDC -> SelectObject ( & pen3 ) ;
    pDC -> MoveTo ( DisplayArea.left, y ) ;
    pDC -> LineTo ( DisplayArea.right, y ) ;
    y += 2 ;

    // loop over the final lines
    for ( i = 0 ; i < m_FinalLines ; i ++ )
        {
        // draw column separators
        int     x = DisplayArea.left ;

        // five in a row
        for ( j = 0 ; j < 5 ; j ++ )
            {
            // start
            x ++ ;
            pDC -> SelectObject ( & pen3 ) ;
            pDC -> MoveTo ( x, y ) ;
            pDC -> LineTo ( x, y + m_Height ) ;
            x += 2 ;

            // save the position
            if ( m_Finals.GetSize () > i * 5 + j )
                {
                m_Finals [ i * 5 + j ].m_X = x ;
                m_Finals [ i * 5 + j ].m_Y = y ;
                }

            // spaces
            for ( k = 0 ; _a_multipliers_f [ k ] ; k ++ )
                {
                // offset
                x += m_Width * _a_multipliers_f [ k ] ;

                // draw
                if ( _a_widths_f [ k ] == 1 )
                    {
                    pDC -> SelectObject ( & pen0 ) ;
                    pDC -> MoveTo ( x, y ) ;
                    pDC -> LineTo ( x, y + m_Height ) ;
                    x ++ ;
                    }
                else
                    {
                    x ++ ;
                    pDC -> SelectObject ( & pen3 ) ;
                    pDC -> MoveTo ( x, y ) ;
                    pDC -> LineTo ( x, y + m_Height ) ;
                    x += 2 ;
                    }
                }// spaces
            }// five in a row

        // offset
        y += m_Height ;

        // separator
        if ( i < m_FinalLines - 1 )
            {
            pDC -> SelectObject ( & pen0 ) ;
            pDC -> MoveTo ( DisplayArea.left, y ) ;
            pDC -> LineTo ( DisplayArea.right, y ) ;
            y ++ ;
            }
        else
            {
            y ++ ;
            pDC -> SelectObject ( & pen3 ) ;
            pDC -> MoveTo ( DisplayArea.left, y ) ;
            pDC -> LineTo ( DisplayArea.right, y ) ;
            y += 2 ;
            }
        }// final lines

    // loop over the display lines
    for ( i = l = m = 0 ; i < m_DispLines ; i ++ )
        {
        // draw column separators
        int     x = DisplayArea.left ;

        // left and right
        for ( j = 0 ; j < 2 ; j ++ )
            {
            // start
            x ++ ;
            pDC -> SelectObject ( & pen3 ) ;
            pDC -> MoveTo ( x, y ) ;
            pDC -> LineTo ( x, y + m_Height ) ;
            x += 2 ;

            // save display position
            if ( m < m_Stands.GetSize () &&
                 m_Stands [ m ].m_IndexX == j &&
                 m_Stands [ m ].m_IndexY == i )
                {
                m_Stands [ m ].m_MainArea.left = x ;
                m_Stands [ m ].m_MainArea.top  = y ;
                m ++ ;
                }

            // spaces
            for ( k = 0 ; _a_multipliers [ k ] ; k ++ )
                {
                // offset
                x += m_Width * _a_multipliers [ k ] ;

                // draw
                if ( _a_widths [ k ] == 1 )
                    {
                    pDC -> SelectObject ( & pen0 ) ;
                    pDC -> MoveTo ( x, y ) ;
                    pDC -> LineTo ( x, y + m_Height ) ;
                    x ++ ;
                    }
                else
                    {
                    x ++ ;
                    pDC -> SelectObject ( & pen3 ) ;
                    pDC -> MoveTo ( x, y ) ;
                    pDC -> LineTo ( x, y + m_Height ) ;
                    x += 2 ;
                    }
                }// spaces
            }// left right

        // offset
        y += m_Height ;

        // separator
        if ( i < m_DispLines - 1 && 
             ( l >= m_SplitLines.GetSize () || i < m_SplitLines [ l ] - 1 ) )
            {
            pDC -> SelectObject ( & pen0 ) ;
            pDC -> MoveTo ( DisplayArea.left, y ) ;
            pDC -> LineTo ( DisplayArea.right, y ) ;
            y ++ ;
            }
        else
            {
            y ++ ;
            pDC -> SelectObject ( & pen3 ) ;
            pDC -> MoveTo ( DisplayArea.left, y ) ;
            pDC -> LineTo ( DisplayArea.right, y ) ;
            y += 2 ;
            l ++ ;
            }
        }// final lines

    // restore
    pDC -> SelectObject ( p_old_pen ) ;
}


//---_DrawGridTexts----------------------------------------------------

void    COccScreen :: _DrawGridTexts ( CDC * pDC,
                                       const CRect & DisplayArea )
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    int         i, j ;
    CFont       font, * p_old_font ;
    CRect       rect1, rect2 ;
    const char  * p_text ;
    UINT        bmid ;
    CSize       size ;
    LOGFONT lf = {
        20,                     // lfHeight; will be calculated later
        0,                      // lfWidth; will be calculated later
        0,                      // LONG lfEscapement; will be calculated later
        0,                      // LONG lfOrientation; will be calculated later
        FW_NORMAL,              // lfWeight;
        false,                  // lfItalic;
        false,                  // lfUnderline;
        false,                  // lfStrikeOut;
#ifdef USE_DEFAULT_CHARSET
        DEFAULT_CHARSET,        // lfCharSet;
#else
        EASTEUROPE_CHARSET,     // lfCharSet;
#endif
        OUT_DEFAULT_PRECIS,     // lfOutPrecision;
        CLIP_DEFAULT_PRECIS,    // lfClipPrecision; 
        DEFAULT_QUALITY,        // lfQuality;
        DEFAULT_PITCH | FF_DONTCARE,    // lfPitchAndFamily
        "Ariel" } ;             // lfFaceName[LF_FACESIZE]


    // update the fields
    if ( m_Height - 1 < 20 )
        lf.lfHeight = m_Height - 1 ;

    // create a new one
    if ( font.CreateFontIndirect ( & lf ) == 0 )
        return ;

    // create a pen
    p_old_font = pDC -> SelectObject ( & font ) ;
    size = pDC -> GetTextExtent ( "Wy", 1 ) ;

    // loop over the final positions
    for ( i = 0 ; i < m_Finals.GetSize () ; i ++ )
        {
        // default
        rect1.top    = m_Finals [ i ].m_Y ;
        rect1.bottom = m_Finals [ i ].m_Y + m_Height - 1 ;

        // set Y coord
        if ( m_Height - 1 < size.cy )
            rect2 = rect1 ;
        else
            {
            rect2.top    = m_Finals [ i ].m_Y + ( m_Height - 1 - size.cy ) / 2 ;
            rect2.bottom = m_Finals [ i ].m_Y + size.cy + 1 ;
            }

        // set starting X
        rect1.left = rect2.left = m_Finals [ i ].m_X ;

        // loop over the boxes
        for ( j = 0 ; _a_multipliers_f [ j ] ; j ++ )
            {
            // set the right side
            rect1.right = rect1.left + m_Width * _a_multipliers_f [ j ] ;
            rect2.right = rect2.left + m_Width * _a_multipliers_f [ j ] ;

            // get the text
            if (( p_text = _GetFinalText ( m_Finals [ i ], j )) != NULL )
                {
                // draw
                pDC -> DrawText ( p_text, & rect2, DT_CENTER ) ;

                // if it is the first then save the AC name
                if ( j == 0 )
                    AddScreenObject ( OBJTYPE_OCCTEXT_CALLSIGN,
                                      p_text,
                                      rect2,
                                      true,
                                      NULL ) ;
                }// text got

            // get the icon
            else if (( bmid = _GetFinalBitmap ( m_Finals [ i ], j )) != 0 )
                {
                CBitmap bm ;


                // load the bitmap
                bm.LoadBitmap ( bmid ) ;

                // draw
                CDC     dc2 ;
                dc2.CreateCompatibleDC ( pDC ) ;
                dc2.SelectObject ( & bm ) ;
                pDC -> StretchBlt ( rect1.left - 2, rect1.top,
                                    rect1.Width (), rect1.Height () + 1,
                                    & dc2,
                                    0, 0, 20, 20,
                                    SRCCOPY ) ;

                // free
                bm.DeleteObject () ;
                dc2.DeleteDC () ;
                }

            // move the X
            rect1.left = rect1.right + _a_widths [ j ] ;
            rect2.left = rect2.right + _a_widths [ j ] ;
            }// loop over the boxes
        }// loop over the stands

    // loop over the stands
    for ( i = 0 ; i < m_Stands.GetSize () ; i ++ )
        {
        // full area
        rect1.top    = m_Stands [ i ].m_MainArea.top ;
        rect1.bottom = m_Stands [ i ].m_MainArea.top + m_Height - 1 ;

        // set Y coord
        if ( m_Height - 1 < size.cy )
            rect2 = rect1 ;
        else
            {
            rect2.top    = m_Stands [ i ].m_MainArea.top + ( m_Height - 1 - size.cy ) / 2 ;
            rect2.bottom = m_Stands [ i ].m_MainArea.top + size.cy + 1 ;
            }

        // set starting X
        rect1.left = rect2.left = m_Stands [ i ].m_MainArea.left ;

        // loop over the boxes
        for ( j = 0 ; _a_multipliers [ j ] ; j ++ )
            {
            // set the right side
            rect1.right = rect1.left + m_Width * _a_multipliers [ j ] ;
            rect2.right = rect2.left + m_Width * _a_multipliers [ j ] ;

            // write back
            if ( j == 0 )
                m_Stands [ i ].m_MainArea = rect1 ;

            // if this is the stand that has the mouse over when
            // dragging a callsign
            if ( j == 0 && m_MovingObjectType && m_MovingOverStandIndex == i )
                {
                // we need a different background
                CRect   rect3 = rect1 ;
                CBrush  br3 ( COLOR_OCC_HILITE ) ;
                pDC -> FillRect ( & rect3, & br3 ) ;
                }

            // get the text
            if (( p_text = _GetStandText ( m_Stands [ i ], j )) != NULL )
                {
                // draw
                pDC -> DrawText ( p_text, & rect2, DT_CENTER ) ;

                // if has objtype
                if ( j == 1 || j == 4 )
                    AddScreenObject ( OBJTYPE_OCCTEXT_CALLSIGN,
                                      p_text,
                                      rect2,
                                      true,
                                      NULL ) ;
                }// text got

            // get the icon
            else if (( bmid = _GetStandBitmap ( m_Stands [ i ], j )) != 0 )
                {
                CBitmap bm ;


                // load the bitmap
                bm.LoadBitmap ( bmid ) ;

                // draw
                CDC     dc2 ;
                dc2.CreateCompatibleDC ( pDC ) ;
                dc2.SelectObject ( & bm ) ;
                pDC -> StretchBlt ( rect1.left, rect1.top,
                                    rect1.Width (), rect1.Height () + 1,
                                    & dc2,
                                    0, 0, 20, 20,
                                    SRCCOPY ) ;

                // free
                bm.DeleteObject () ;
                dc2.DeleteDC () ;
                }

            // move the X
            rect1.left = rect1.right + _a_widths [ j ] ;
            rect2.left = rect2.right + _a_widths [ j ] ;
            }// loop over the boxes
        }// loop over the stands

    // moving object
    if ( m_MovingObjectType == OBJTYPE_OCCTEXT_CALLSIGN )
        pDC -> DrawText ( m_MovingObjectId, & m_MovingArea, DT_CENTER ) ;

    // restore
    pDC -> SelectObject ( p_old_font ) ;
}


//---_SearchForAllocation----------------------------------------------

int     COccScreen :: _SearchForAllocation ( const char * sCallsign )
{
    int     i, j ;


    // loop over the stands
    for ( i = 0 ; i < m_Stands.GetSize () ; i ++ )
        {
        // loop over the allocations
        for ( j = 0 ; j < m_Stands [ i ].m_Allocations.GetSize () ; j ++ )
            {
            // if found
            if ( m_Stands [ i ].m_Allocations [ j ].m_Callsign.CompareNoCase ( sCallsign ) == 0 )
                return i ;
            }// loop over the allocations
        }// loop over the stands

    // not found
    return -1 ;
}


//---_GetFinalText-----------------------------------------------------

const char *    COccScreen :: _GetFinalText ( const COccFinalPositions & Final,
                                              int Index )
{
    static CString                  fmt ;
    EuroScopePlugIn :: CAircraft    ac ;

    
    // if nothing is registered to the position
    if ( Final.m_ACPosition == NULL )
        return NULL ;

    // get the AC
    COccAircraft    & occac = m_VinicityAcList.GetAt ( Final.m_ACPosition ) ;

    // switch by the index
    switch ( Index )
        {
        case 0 :    // the gate name
            return occac.m_Callsign ;

        case 1 : // the icon
            return NULL ;

        case 2 : // allocate gate if found

            // get the AC
            ac = GetPlugIn () -> AircraftSelect ( occac.m_Callsign ) ;
            if ( ! ac.IsValid ())
                return NULL ;

            // type in every second
            if (( time ( NULL ) % 2 ) == 0 && ac.GetFlightPlan ().IsReceived ())
                return ac.GetFlightPlan ().GetAircraftFPType () ;

            // then the gate or distance
            if ( Final.m_GateIndex >= 0 )
                return m_Stands [ Final.m_GateIndex ].m_Name ;
            else
                {
                // distance
                fmt.Format ( "%.0f", ac.GetDistanceToDestination ()) ;
                return fmt ;
                }

            return NULL ;
        }// switch by the index

    // no text
    return NULL ;
}


//---_GetStandText-----------------------------------------------------

const char *    COccScreen :: _GetStandText ( const COccStand & Stand,
                                              int Index )
{
    static CString                  fmt ;
    long                            mins ;
    EuroScopePlugIn :: CAircraft    ac ;


    // switch by the index
    switch ( Index )
        {
        case 0 :    // the gate name
            return Stand.m_Name ;

        case 1 : // the AC at the gate

            if ( Stand.m_ACsAtGate.GetSize ())
                {
                // get the AC
                COccAircraft    & occac = m_VinicityAcList.GetAt ( Stand.m_ACsAtGate [ 0 ] ) ;

                // return the callsign
                return occac.m_Callsign ;
                }

            return NULL ;

        case 2 : // icon
            return NULL ;

        case 3 : // how long it stays there

            if ( Stand.m_ACsAtGate.GetSize ())
                {
                // get the AC
                COccAircraft    & occac = m_VinicityAcList.GetAt ( Stand.m_ACsAtGate [ 0 ] ) ;

                // calculate
                mins = (int) ( time ( NULL ) - occac.m_StateChanged ) ;
                fmt.Format ( "%02d:%02d", mins / 60, mins % 60 ) ;
                return fmt ;
                }

            return NULL ;

        case 4 : // second callsign

            if ( Stand.m_Allocations.GetSize () == 0 )
                return NULL ;

            return Stand.m_Allocations [ 0 ].m_Callsign ;

        case 5 : // icon

            return NULL ;

        case 6 :

            // if no AC
            if ( Stand.m_Allocations.GetSize () == 0 )
                return NULL ;

            // if departing
            if ( Stand.m_Allocations [ 0 ].m_Departure )
                {
                mins = (int) ( time ( NULL ) - Stand.m_Allocations [ 0 ].m_Allocated ) ;
                fmt.Format ( "%02d:%02d", mins / 60, mins % 60 ) ;
                return fmt ;
                }

            // arriving
            else
                {
                // if taxiing
                POSITION    pos = _SelectAcFromVinicityList ( Stand.m_Allocations [ 0 ].m_Callsign ) ;
                if ( pos &&
                     m_VinicityAcList.GetAt ( pos ).m_State == OCC_STATE_TAXIING_TO_GATE )
                    fmt = "TAXI" ;

                // distance
                else
                    {
                    // get the AC
                    ac = GetPlugIn () -> AircraftSelect ( Stand.m_Allocations [ 0 ].m_Callsign ) ;
                    if ( ! ac.IsValid ())
                        return NULL ;
                    fmt.Format ( "%.0f", ac.GetDistanceToDestination ()) ;
                    }

                return fmt ;
                }

        case 7 :

            // if no AC
            if ( Stand.m_Allocations.GetSize () < 2 )
                return NULL ;

            // get the AC
            ac = GetPlugIn () -> AircraftSelect ( Stand.m_Allocations [ 1 ].m_Callsign ) ;
            if ( ! ac.IsValid ())
                return NULL ;

                // distance
            fmt.Format ( "%.0f", ac.GetDistanceToDestination ()) ;
            return fmt ;

        }// switch by the index

    // no text
    return NULL ;
}


//---_GetStandBitmap---------------------------------------------------

UINT    COccScreen :: _GetStandBitmap ( const COccStand & Stand,
                                        int Index )
{
    // switch by the index
    switch ( Index )
        {
        case 2 :

            // is there an AC there?
            if ( Stand.m_ACsAtGate.GetSize ())
                {
                // get the AC
                COccAircraft    & occac = m_VinicityAcList.GetAt ( Stand.m_ACsAtGate [ 0 ] ) ;

                // if assigned
                if ( occac.m_AcceptedAtGate )
                    {
                    if ( occac.m_State == OCC_STATE_AT_GATE_DEPARTING )
                        return IDB_KEK_FELSZALL_BITMAP ;
                    else if ( occac.m_State == OCC_STATE_AT_GATE_PUSHING_BACK )
                        return IDB_KEK_PUSHBACK_BITMAP ;
                    else
                        return IDB_ZOLD_LESZALL_BITMAP ;
                    }
                else
                    {
                    if ( occac.m_State == OCC_STATE_AT_GATE_DEPARTING )
                        return IDB_PIROS_FELSZALL_BITMAP ;
                    else if ( occac.m_State == OCC_STATE_AT_GATE_PUSHING_BACK )
                        return IDB_PIROS_PUSHBACK_BITMAP ;
                    else
                        return IDB_PIROS_LESZALL_BITMAP ;
                    }
                }

            return 0 ;


        case 5 :

            // has allocation
            if ( Stand.m_Allocations.GetSize () == 0 )
                return 0 ;

            // departing
            if ( Stand.m_Allocations [ 0 ].m_Departure )
                return IDB_KEK_FELSZALL_BITMAP ;

            // arriving
            else
                {
                // search for it in the vinicity
                POSITION    pos = _SelectAcFromVinicityList ( Stand.m_Allocations [ 0 ].m_Callsign ) ;
                if ( pos &&
                     m_VinicityAcList.GetAt ( pos ).m_State == OCC_STATE_TAXIING_TO_GATE )
                    return IDB_ZOLD_LEFELE_BITMAP ;
                else
                    return IDB_ZOLD_LESZALL_BITMAP ;
                }

        }// switch by the index

    // no icon
    return 0 ;
}


//---_GetFinalBitmap---------------------------------------------------

UINT    COccScreen :: _GetFinalBitmap ( const COccFinalPositions & Final,
                                        int Index )
{
    // if nothing is registered to the position
    if ( Final.m_ACPosition == NULL )
        return 0 ;

    // only for index 1
    if ( Index != 1 )
        return 0 ;

    // get the AC
    COccAircraft    & occac = m_VinicityAcList.GetAt ( Final.m_ACPosition ) ;

    // if arriving
    if ( occac.m_State == OCC_STATE_ON_FINAL )
        {
        if ( Final.m_GateIndex >= 0 )
            return IDB_ZOLD_LESZALL_BITMAP ;
        else
            return IDB_PIROS_LESZALL_BITMAP ;
        }

    // on the ground
    else
        {
        if ( Final.m_GateIndex >= 0 )
            return IDB_ZOLD_LEFELE_BITMAP ;
        else
            return IDB_PIROS_LEFELE_BITMAP ;
        }

    // no icon
    return 0 ;
}


//---_SelectAcFromVinicityList-----------------------------------------

POSITION    COccScreen :: _SelectAcFromVinicityList ( const char * sCallsign )
{
    POSITION    pos ;


    // search
    pos = m_VinicityAcList.GetHeadPosition () ;
    while ( pos )
        {
        // if found
        if ( m_VinicityAcList.GetAt ( pos ).m_Callsign.CompareNoCase ( sCallsign ) == 0 )
            return pos ;

        // next
        m_VinicityAcList.GetNext ( pos ) ;
        }

    // not found
    return NULL ;
}


//---_TestIfCoordAtGate------------------------------------------------

int     COccScreen :: _TestIfCoordAtGate ( const EuroScopePlugIn :: CPosition & Coord )
{
    int     i, j, gate = -1 ;
    double  dist, min_dist = 100 ;


    // search for the gates
    for ( i = 0 ; i < m_Stands.GetSize () ; i ++ )
        {
        // serach for coords
        for ( j = 0 ; j < 2 ; j ++ )
            {
            // if the coords are not set
            if ( ! m_Stands [ i ].m_aCoordsSet [ j ] )
                continue ;

            // calculate the distance
            dist = Coord.DistanceTo ( m_Stands [ i ].m_aCoords [ j ] ) ;

            // if not close enough
            if ( dist > 0.05 )
                continue ;

            // if first or closer
            if ( min_dist < 0 || dist < min_dist )
                {
                gate     = i ;
                min_dist = dist ;
                }
            }// search for the coords
        }// search for the gates

    // found or not
    return gate ;
}


//---_TestIfAllocatedToHere--------------------------------------------

bool    COccScreen :: _TestIfAllocatedToHere ( const char * sCallsign,
                                               int GateIndex )
{
    int     i ;
    bool    res = false ;


    // test if allocated to here
    for ( i = 0 ;
          i < m_Stands [ GateIndex ].m_Allocations.GetSize () &&
              m_Stands [ GateIndex ].m_Allocations [ i ].m_Callsign.CompareNoCase ( sCallsign ) &&
              m_Stands [ GateIndex ].m_Allocations [ i ].m_Callsign.CompareNoCase ( "*" ) ;
          i ++ ) ;

    // if allocated
    if ( i < m_Stands [ GateIndex ].m_Allocations.GetSize ())
        {
        // accept
        res = true ;

        // delete the allocation
        m_Stands [ GateIndex ].m_Allocations.RemoveAt ( i ) ;
        }// gate is allocated

    return res ;
}

//---_ArriveToGate-----------------------------------------------------

void    COccScreen :: _ArriveToGate ( const char * sGateNumber,
                                      const char * pCallsign )
{
    int             idx ;
    CString         call = pCallsign ;
    COccAllocation  alloc ;


    // search for the gate
    if (( idx = _SelectGate ( sGateNumber )) < 0 )
        return ;

    // delete all prev allocations
    _DeleteAllocations ( call ) ;

    // save
    alloc.m_Departure = false ;
    alloc.m_Callsign  = call ;
    alloc.m_Callsign.MakeUpper () ;
    alloc.m_Allocated = time ( NULL ) ;
    m_Stands [ idx ].m_Allocations.Add ( alloc ) ;
}


//---_DepartFromGate---------------------------------------------------

void    COccScreen :: _DepartFromGate ( const char * sGateNumber,
                                        const char * pCallsign )
{
    int             idx, i = 0 ;
    CString         call = pCallsign ;
    COccAllocation  alloc ;


    // search for the gate
    if (( idx = _SelectGate ( sGateNumber )) < 0 )
        return ;

    // uppercase
    call.MakeUpper () ;

    // delete all prev allocations
    _DeleteAllocations ( call ) ;

    // fill the struct
    alloc.m_Departure = true ;
    alloc.m_Callsign  = call ;
    alloc.m_Callsign.MakeUpper () ;
    alloc.m_Allocated = time ( NULL ) ;

    // if the first is a departure
    if ( m_Stands [ idx ].m_Allocations.GetSize () &&
         m_Stands [ idx ].m_Allocations [ 0 ].m_Departure )
        m_Stands [ idx ].m_Allocations [ 0 ] = alloc ;

    // else insert to the beginning
    else if ( m_Stands [ idx ].m_Allocations.GetSize ())
        m_Stands [ idx ].m_Allocations.InsertAt ( 0, alloc ) ;

    else
        m_Stands [ idx ].m_Allocations.Add ( alloc ) ;
}


//---_SelectGate-------------------------------------------------------

int     COccScreen :: _SelectGate ( const char * sGateNumber )
{
    int     i ;


    // searc
    for ( i = 0 ; i < m_Stands.GetSize () ; i ++ )
        {
        // if empty
        if ( m_Stands [ i ].m_Name.IsEmpty ())
            continue ;

        // if matches
        if ( strcmp ( m_Stands [ i ].m_GateNumber, sGateNumber ) == 0 )
            return i ;
        }

    // not found
    return -1 ;
}


//---_DeleteAllocations------------------------------------------------

void    COccScreen :: _DeleteAllocations ( const char * sCallsign )
{
    int     i, j ;


    // do not delete * allocations
    if ( sCallsign [ 0 ] == '*' )
        return ;

    // loop over the stands
    for ( i = 0 ; i < m_Stands.GetSize () ; i ++ )
        {
        // loop over the allocations
        for ( j = 0 ; j < m_Stands [ i ].m_Allocations.GetSize () ; j ++ )
            {
            // if found
            if ( m_Stands [ i ].m_Allocations [ j ].m_Callsign.CompareNoCase ( sCallsign ) == 0 )
                m_Stands [ i ].m_Allocations.RemoveAt ( j -- ) ;
            }// loop over the allocations
        }// loop over the stands
}

