#pragma once

#include "EuroScopePlugIn.h"

const   int     OCC_STATE_AT_GATE_DEPARTING     = 1 ;
const   int     OCC_STATE_AT_GATE_PUSHING_BACK  = 2 ;
const   int     OCC_STATE_TAXIING_TO_RW         = 3 ;
const   int     OCC_STATE_TAKING_OFF            = 4 ;

const   int     OCC_STATE_ON_FINAL              = 11 ;
const   int     OCC_STATE_TAXIING_TO_GATE       = 12 ;
const   int     OCC_STATE_AT_GATE_ARRIVED       = 13 ;


class CESRect
{
public:

    EuroScopePlugIn :: CPosition    m_LeftDown ;
    EuroScopePlugIn :: CPosition    m_RightUp ;

} ;

class COccAllocation
{
public:

    bool        m_Departure ;               // allocated for departure?

    CString     m_Callsign ;                            // the callsign
    time_t      m_Allocated ;                  // when it was allocated

} ;


class COccAircraft
{
public:

    CString     m_Callsign ;                  // the callsign of the AC

    int         m_State ;                        // the state of the AC

    double      m_Dist ;                                    // distance
    int         m_GateIndex ;             // the close enogh gate index
    time_t      m_StateChanged ;              // the last state changed

    bool        m_AcceptedAtGate ;
                           // indicates that accepted its gate position


    //---SetNewState---------------------------------------------------

    inline  void    SetNewState ( int NewState )
        {
        m_State = NewState ;
        m_StateChanged = time ( NULL ) ;
        } ;

} ;


class COccFinalPositions
{
public:

    int         m_X, m_Y ;                          // display position

    POSITION    m_ACPosition ;              // the AC to be filled here

    int         m_GateIndex ;          // the gate allocated for the AC

} ;


class COccStand
{
public:

    int         m_IndexX, m_IndexY ;                         // indexes
    CRect       m_MainArea ;       // the main area of the stand number

    CString     m_GateNumber ;         // the number format of the GATE
    CString     m_Name ;                                        // name

    EuroScopePlugIn :: CPosition    m_aCoords [ 2 ] ;
                                // the two possible coords of the stand
                        // unfortunately we got two different scenarios
    bool        m_aCoordsSet [ 2 ] ;   // indicates that coords are set

    CArray < COccAllocation, COccAllocation & >
                m_Allocations ;                      // the allocations

    CArray < POSITION, POSITION >
                m_ACsAtGate ;                         // the ACs at the


    //---COccStand-----------------------------------------------------

    COccStand ( void )
        {
        m_aCoordsSet [ 0 ] = m_aCoordsSet [ 1 ] = false ;
        } ;
    COccStand ( const COccStand & Other )
        {
        * this = Other ;
        } ;


    //---operator =----------------------------------------------------

    inline  COccStand & operator = ( const COccStand & Other )
        {
        m_IndexX     = Other.m_IndexX ;
        m_IndexY     = Other.m_IndexY ;
        m_MainArea   = Other.m_MainArea ;
        m_GateNumber = Other.m_GateNumber ;
        m_Name       = Other.m_Name ;
        m_aCoords [ 0 ]    = Other.m_aCoords [ 0 ] ;
        m_aCoords [ 1 ]    = Other.m_aCoords [ 1 ] ;
        m_aCoordsSet [ 0 ] = Other.m_aCoordsSet [ 0 ] ;
        m_aCoordsSet [ 1 ] = Other.m_aCoordsSet [ 1 ] ;

        return * this ;
        } ;

} ;


class COccScreen :
    public EuroScopePlugIn :: CRadarScreen
{
protected:

    bool        m_Valid ;           // indicates a valid, loaded screen

    CArray < COccStand, COccStand & > m_Stands ;    // the stands/gates
    CArray < COccFinalPositions, COccFinalPositions & >
                m_Finals ;                           // final positions

    int         m_FinalLines ;             // the number of final lines
    int         m_DispLines ;    // the number of lines to be displayed
    int         m_TopEmpty ;                       // empty area on top
    int         m_BottomEmpty ;                 // empty area on bottom
    int         m_LeftEmpty, m_RightEmpty ;         // side empty areas

    CArray < int, int >
                m_SplitLines ;              // where to draw wide lines

    int         m_Width, m_Height ;                 // block base sizes

    CESRect     m_AirportExtent ;             // the extent of the area
    CESRect     m_AirportBiggerExtent ;            // the arriving area

    CString     m_AirportName ;         // the destination airport name

    CList < COccAircraft, COccAircraft & >
                m_VinicityAcList ;            // the ACs in the vincity

    int         m_MovingObjectType ;
    CString     m_MovingObjectId ;
    CRect       m_MovingArea ;                     // the moving object
    int         m_MovingOverStandIndex ;

public:

    //---COccScreen----------------------------------------------------

    COccScreen ( void ) ;


    //---~COccScreen---------------------------------------------------

    ~COccScreen ( void ) ;


    //---OnAsrContentLoaded--------------------------------------------

    virtual void    OnAsrContentLoaded ( bool Loaded ) ;
    //------------------------------------------------------------------
    //  When data is fully loaded.
    //------------------------------------------------------------------


    //---OnRefresh------------------------------------------------------

    virtual void    OnRefresh ( HDC hDC,
                                int Phase ) ;
    //------------------------------------------------------------------
    //  The screen refresh function.
    //------------------------------------------------------------------


    //---OnAircraftPositionUpdate--------------------------------------

    virtual void    OnAircraftPositionUpdate ( EuroScopePlugIn :: CAircraft Aircraft ) ;
    //-----------------------------------------------------------------
    //  The plane position update message.
    //-----------------------------------------------------------------


    //---OnCompileCommand-----------------------------------------------

    virtual bool    OnCompileCommand ( const char * sCommandLine ) ;
    //-----------------------------------------------------------------
    //  The command compiler.
    //-----------------------------------------------------------------


    //---OnMoveScreenObject---------------------------------------------

    virtual void    OnMoveScreenObject ( int ObjectType,
                                         const char * sObjectId,
                                         POINT Pt,
                                         RECT Area,
                                         bool Released ) ;
    //-----------------------------------------------------------------
    //  The screen item mover.
    //-----------------------------------------------------------------


    //---OnAsrContentToBeClosed-----------------------------------------

    inline  virtual void    OnAsrContentToBeClosed ( void )
        {
        delete this ;
        } ;

protected:

    //---_DrawGridBorders----------------------------------------------

    void    _DrawGridBorders ( CDC * pDC,
                               const CRect & DisplayArea ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  pDC         - the device context
    //      =>  DisplayArea - the area for the screen
    //
    //  Description :
    //      It prints the grid border lines.
    //-----------------------------------------------------------------


    //---_DrawGridTexts------------------------------------------------

    void    _DrawGridTexts ( CDC * pDC,
                             const CRect & DisplayArea ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  pDC     - the device context
    //      =>  DisplayArea - the area for the screen
    //
    //  Description :
    //      It prints the grid texts.
    //-----------------------------------------------------------------


    //---_SearchForAllocation------------------------------------------

    int     _SearchForAllocation ( const char * sCallsign ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  sCallsign   - the AC callsign
    //
    //  Return :
    //      the index of the gate or
    //      -1 if not allocated to anywhere
    //
    //  Description :
    //      It searches for the first allocation.
    //-----------------------------------------------------------------


    //---_GetFinalText-------------------------------------------------

    const char *    _GetFinalText ( const COccFinalPositions & Final,
                                    int Index ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  Stand       - the stand itsef
    //      =>  Index       - the index of the text to get
    //
    //  Return :
    //      the text to be displayed.
    //-----------------------------------------------------------------


    //---_GetStandText-------------------------------------------------

    const char *    _GetStandText ( const COccStand & Stand,
                                    int Index ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  Stand       - the stand itsef
    //      =>  Index       - the index of the text to get
    //
    //  Return :
    //      the text to be displayed.
    //-----------------------------------------------------------------


    //---_GetStandBitmap-----------------------------------------------

    UINT    _GetStandBitmap ( const COccStand & Stand,
                              int Index ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  Stand   - the stand itsef
    //      =>  Index   - the index of the text to get
    //
    //  Return :
    //      the resource id of the bitmap.
    //-----------------------------------------------------------------


    //---_GetFinalBitmap-----------------------------------------------

    UINT    _GetFinalBitmap ( const COccFinalPositions & Final,
                              int Index ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  Stand   - the stand itsef
    //      =>  Index   - the index of the text to get
    //
    //  Return :
    //      the resource id of the bitmap.
    //-----------------------------------------------------------------


    //---_SelectAcFromVinicityList-------------------------------------

    POSITION    _SelectAcFromVinicityList ( const char * sCallsign ) ;
    //-----------------------------------------------------------------
    //  Parameter :
    //      =>  sCallsign   - the AC's callsign
    //
    //  Return :
    //      the position in the list
    //      or NULL if not found
    //-----------------------------------------------------------------


    //---_TestIfCoordAtGate--------------------------------------------

    int     _TestIfCoordAtGate ( const EuroScopePlugIn :: CPosition & Coord ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  Coord   - the coordinate
    //
    //  Return :
    //      - the index of the gate
    //      - -1 if no gate close enough
    //
    //  Description :
    //      It searches for the closest gate.
    //-----------------------------------------------------------------


    //---_TestIfAllocatedToHere----------------------------------------

    bool    _TestIfAllocatedToHere ( const char * sCallsign,
                                     int GateIndex ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  sCallsign   - the AC callsign
    //      =>  GateIndex   - the index of the gate to be tested
    //
    //  Return :
    //      true    - if there is an allocation for the AC
    //      false   - if no allocation for the AC
    //
    //  Description :
    //      It tests if there is an allocation for the AC at the gate.
    //      It also deletes the allocation immediately.
    //-----------------------------------------------------------------


    //---_ArriveToGate-------------------------------------------------

    void    _ArriveToGate ( const char * sGateNumber,
                            const char * pCallsign ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  sGateNumber - the gate to be allocated
    //      =>  pCallsign   - the callsign
    //
    //  Description :
    //      It allocates the gate for an arrival.
    //-----------------------------------------------------------------


    //---_DepartFromGate-----------------------------------------------

    void    _DepartFromGate ( const char * sGateNumber,
                              const char * pCallsign ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  sGateNumber - the gate to be allocated
    //      =>  pCallsign   - the callsign
    //
    //  Description :
    //      It allocates the gate for an departure.
    //-----------------------------------------------------------------


    //---_SelectGate---------------------------------------------------

    int     _SelectGate ( const char * sGateNumber ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  sGateNumber - the gate we are searching for
    //
    //  Return :
    //      - the gate index
    //      - or -1 if not found
    //
    //  Description :
    //      It selects the gate by number.
    //-----------------------------------------------------------------


    //---_DeleteAllocations--------------------------------------------

    void    _DeleteAllocations ( const char * sCallsign ) ;
    //-----------------------------------------------------------------
    //  Parameters :
    //      =>  sCallsign   - the AC callsign
    //
    //  Description :
    //      It deletes all allocations for the AC.
    //-----------------------------------------------------------------


};
